from typing import Iterable, Optional

from fetchfox.checks import check_str
from .common import get, post


def get_orders(asset_id: str, partner_code: str = None) -> Iterable[dict]:
    check_str(asset_id, "dexhunterio.asset_id")

    body = {
        "filters": [
            {
                "filterType": "TOKENID",
                "values": [asset_id],
            },
            {
                "filterType": "STATUS",
                "values": ["COMPLETED"],
            },
        ],
        "orderSorts": "STARTTIME",
        "page": 0,
        "perPage": 100,
        "sortDirection": "DESC",
    }

    page = -1

    while True:
        page += 1
        body["page"] = page

        response = post(
            "swap/globalOrders",
            body=body,
            partner_code=partner_code,
        )

        if not response.get("orders"):
            break

        yield from response["orders"]


def get_average_price(asset_id: str, partner_code: str = None) -> Optional[float]:
    check_str(asset_id, "dexhunterio.asset_id")

    response = get(
        service=f"swap/averagePrice/{asset_id}/ADA",
        partner_code=partner_code,
    )

    if response is None:
        return None

    return response.get("price_ba")


def get_pair_stats(asset_id: str, partner_code: str = None) -> dict:
    check_str(asset_id, "dexhunterio.asset_id")

    return get(
        service=f"swap/pairStats/{asset_id}/ADA",
        partner_code=partner_code,
    )



from typing import Iterable, Tuple

from fetchfox.apis.cardano import gomaestroorg


class CardanoPool:
    def __init__(self, id: str, name: str, fingerprint: str, gomaestroorg_api_key: str = None):
        self.id = id
        self.name = name
        self.fingerprint = fingerprint
        self.gomaestroorg_api_key = gomaestroorg_api_key

    @property
    def delegated_amount(self) -> float:
        info = gomaestroorg.get_pool_information(
            self.fingerprint,
            api_key=self.gomaestroorg_api_key,
        )

        return info["live_stake"] / 10**6

    @property
    def delegator_count(self) -> int:
        info = gomaestroorg.get_pool_information(
            self.fingerprint,
            api_key=self.gomaestroorg_api_key,
        )

        return info["live_delegators"]

    @property
    def saturation(self) -> float:
        info = gomaestroorg.get_pool_information(
            self.fingerprint,
            api_key=self.gomaestroorg_api_key,
        )

        return float(info["live_saturation"])

    @property
    def delegators(self) -> Iterable[Tuple[str, float]]:
        yield from gomaestroorg.get_pool_delegators(
            self.fingerprint,
            api_key=self.gomaestroorg_api_key,
        )

    @property
    def url(self) -> str:
        return f"https://pool.pm/{self.id}"

    @property
    def __repr__(self):
        return f"{self.name} ({self.id})"



from fetchfox.constants.algorand.basics import ALGO
from fetchfox.constants.cardano.basics import ADA
from fetchfox.constants.cardano.tokens.beard import BEARD_TOKEN_SYMBOL as BEARD
from fetchfox.constants.cardano.tokens.book import BOOK_TOKEN_SYMBOL as BOOK
from fetchfox.constants.cardano.tokens.charles import CHARLES_TOKEN_SYMBOL as CHARLES
from fetchfox.constants.cardano.tokens.stuff import STUFF_TOKEN_SYMBOL as STUFF
from fetchfox.constants.ethereum.basics import ETH
from fetchfox.constants.fiat import USD
from fetchfox.constants.polygon.basics import MATIC, POL

CRYPTOS = [
    # cryptocurrencies
    ALGO,
    ADA,
    ETH,
    MATIC,
    POL,
    # cardano tokens
    BEARD,
    BOOK,
    CHARLES,
    STUFF,
]

FIATS = [
    USD,
]



BEARD_TOKEN_SYMBOL = "BEARD"

BEARD_TOKEN_POLICY_ID = "8d7526784ef72fe0ccdd085976ada0da88e7fb013e38e794b0923341"
BEARD_TOKEN_ASSET_NAME = "4245415244"

BEARD_TOKEN_ASSET_ID = f"{BEARD_TOKEN_POLICY_ID}{BEARD_TOKEN_ASSET_NAME}"

BEARD_TOKEN_FINGERPRINT = "asset1kkuscpr9fg4r038te6ue93eskht76gmylmsav9"



from typing import Iterable

from fetchfox.checks import check_str
from .common import get
from .schemas import PoolInformationSchema, PoolDelegatorSchema


def get_information(pool_id: str, api_key: str = None, preprod: bool = False) -> PoolInformationSchema:
    check_str(pool_id, "gomaestroorg.pool_id")

    pool_id = pool_id.strip().lower()

    response, status_code = get(
        service=f"pools/{pool_id}/info",
        api_key=api_key,
        preprod=preprod,
    )

    return PoolInformationSchema.model_validate(response["data"])


def get_delegators(pool_id: str, api_key: str = None, preprod: bool = False) -> Iterable[PoolDelegatorSchema]:
    check_str(pool_id, "gomaestroorg.pool_id")

    pool_id = pool_id.strip().lower()

    cursor = None

    while True:
        response, status_code = get(
            service=f"pools/{pool_id}/delegators",
            params={
                "cursor": cursor,
            },
            api_key=api_key,
            preprod=preprod,
            check="data",
        )

        for item in response.get("data", []):
            delegator = PoolDelegatorSchema.model_validate(item)

            if delegator.amount == 0:
                continue

            yield delegator

        cursor = response.get("next_cursor")

        if not cursor:
            break



class InvalidCollectionIdException(ValueError):
    def __init__(self, string: str):
        super().__init__(string)


class InvalidAssetIdException(ValueError):
    def __init__(self, string: str):
        super().__init__(string)


class InvalidWalletException(ValueError):
    def __init__(self, string: str):
        super().__init__(string)



JPG_STORE_STAKE_ADDRESSES = [
    "stake1uxqh9rn76n8nynsnyvf4ulndjv0srcc8jtvumut3989cqmgjt49h6",
    "stake17ykfvl6t62y5fvryvtsnch3lt406dcpls4n4d9pcekpnumg6v83tq",
]

MINSWAP_ORG_STAKE_ADDRESSES = [
    "stake1u9f9v0z5zzlldgx58n8tklphu8mf7h4jvp2j2gddluemnssjfnkzz",
]



import logging
import os
from typing import Tuple

from fetchfox import rest
from fetchfox.checks import check_str
from fetchfox.constants.blockchains import ETHEREUM, POLYGON

BASE_URL_DEFAULT = "https://api.opensea.io"
BASE_URL = os.getenv("OPENSEAIO_BASE_URL") or BASE_URL_DEFAULT

API_KEY = os.getenv("OPENSEAIO_API_KEY")

BLOCKCHAINS = {
    ETHEREUM: "ethereum",
    POLYGON: "matic",
}

logger = logging.getLogger(__name__)


def get(service: str, params: dict = None, version: int = 2, check: str = None, api_key: str = None) -> Tuple[dict, int]:
    api_key = api_key or API_KEY
    check_str(api_key, "openseaio.api_key")

    if version == 1:
        url = f"{BASE_URL}/api/v{version}/{service}"
    else:
        url = f"{BASE_URL}/v{version}/{service}"

    return rest.get(
        url=url,
        headers={
            "X-API-KEY": api_key,
        },
        params=params,
        sleep=2.5,
        check=check,
    )



from typing import Iterable

from fetchfox.checks import check_str
from .common import get
from .schemas import CreatorListingSchema


def get_listings(creator_address: str) -> Iterable[CreatorListingSchema]:
    check_str(creator_address, "randswapcom.creator_address")
    creator_address = creator_address.strip().upper()

    response, status_code = get(
        service=f"listings/creator/{creator_address}",
    )

    yield from map(
        CreatorListingSchema.model_validate,
        response,
    )



from functools import lru_cache
from typing import Iterable

from fetchfox.checks import check_str
from .common import get
from .schemas import AssetHolderSchema


def get_holders(asset_id: str, api_key: str = None, preprod: bool = False) -> Iterable[AssetHolderSchema]:
    check_str(asset_id, "gomaestroorg.asset_id")

    cursor = None

    while True:
        response, status_code = get(
            service=f"assets/{asset_id}/accounts",
            params={
                "cursor": cursor,
            },
            api_key=api_key,
            preprod=preprod,
        )

        for data in response.get("data", []):
            data["asset_id"] = asset_id
            yield AssetHolderSchema.model_validate(data)

        cursor = response.get("next_cursor")

        if not cursor:
            break


@lru_cache(maxsize=None)
def get_data(asset_id: str, api_key: str = None, preprod: bool = False) -> dict:
    check_str(asset_id, "gomaestroorg.asset_id")

    asset_id = asset_id.strip().lower()

    response, status_code = get(
        service=f"assets/{asset_id}",
        api_key=api_key,
        preprod=preprod,
        check="data",
    )

    cip25_metadata = response["data"]["asset_standards"].get("cip25_metadata")
    cip68_metadata = response["data"]["asset_standards"].get("cip68_metadata")

    metadata = cip68_metadata or cip25_metadata
    metadata["total_supply"] = response["data"].get("total_supply", "1")

    return metadata



import logging
from functools import lru_cache
from typing import Iterable

from fetchfox.checks import check_str
from .common import get, BLOCKCHAINS
from .schemas import CollectionsListingsSchema

logger = logging.getLogger(__name__)


@lru_cache(maxsize=None)
def get_slug(contract_address: str, blockchain: str, api_key: str = None) -> str:
    check_str(contract_address, "openseaio.contract_address")
    contract_address = contract_address.strip().lower()

    check_str(blockchain, "openseaio.blockchain")
    blockchain: str = BLOCKCHAINS.get(blockchain, blockchain)

    logger.info("fetching slug for %s (%s)", contract_address, blockchain)

    response, status_code = get(
        service=f"chain/{blockchain}/contract/{contract_address}",
        api_key=api_key,
        check="collection",
    )

    return response["collection"]


def get_listings(contract_address: str, blockchain: str, slug: str = None, api_key: str = None) -> Iterable[CollectionsListingsSchema]:
    check_str(contract_address, "openseaio.contract_address")
    contract_address = contract_address.strip().lower()

    check_str(blockchain, "openseaio.blockchain")

    if not slug:
        slug = get_slug(contract_address, blockchain, api_key=api_key)

    cursor = ""

    while True:
        response, status_code = get(
            service=f"listings/collection/{slug}/all",
            params={
                "next": cursor,
            },
            api_key=api_key,
        )

        yield from map(
            CollectionsListingsSchema.model_validate,
            response.get("listings", []),
        )

        cursor = response.get("next")

        if not cursor:
            break



class StatsDTO:
    def __init__(
        self,
        ada: float,
        amount: float,
        symbol: str,
        daily_txs: int,
        daily_buys: int,
        daily_sales: int,
        daily_volume: float,
        price_change_day: float,
    ):
        self.ada: float = ada
        self.amount: float = amount
        self.symbol: str = symbol

        self.daily_txs: int = daily_txs
        self.daily_buys: int = daily_buys
        self.daily_sales: int = daily_sales
        self.daily_volume: float = daily_volume
        self.price_change_day: float = price_change_day

    def __repr__(self):
        return f"{round(self.ada)} ADA - {round(self.amount)} {self.symbol}"


class PairStatsDTO:
    def __init__(self, stats: StatsDTO):
        self.stats: StatsDTO = stats

    def __repr__(self):
        return f"{self.stats}"



class HoldingDTO:
    def __init__(self, collection_id: str, asset_id: str, address: str, amount: float):
        self.collection_id: str = collection_id
        self.asset_id: str = asset_id
        self.address: str = address
        self.amount: float = amount

    def __repr__(self) -> str:
        if not self.collection_id:
            return f"{self.address}: {self.asset_id} x {self.amount}"

        return f"{self.address}: {self.collection_id}/{self.asset_id} x {self.amount}"



CHARLES_TOKEN_SYMBOL = "CHARLES"

CHARLES_TOKEN_POLICY_ID = "c8c711a3df70956c397cb92e1705b675b18652723cfb7f72ccd59f2d"
CHARLES_TOKEN_ASSET_NAME = "434841524c4553"

CHARLES_TOKEN_ASSET_ID = f"{CHARLES_TOKEN_POLICY_ID}{CHARLES_TOKEN_ASSET_NAME}"

CHARLES_TOKEN_FINGERPRINT = "asset1849ky50sq5gudwfn368dznzqpwfrsq7q5fvw2y"



from fetchfox.constants.cardano.pools.stuff import STUFF_POOL_FINGERPRINT, STUFF_POOL_ID, STUFF_POOL_NAME
from .base import CardanoPool


class StuffPool(CardanoPool):
    def __init__(self, gomaestroorg_api_key: str = None):
        super().__init__(
            id=STUFF_POOL_ID,
            name=STUFF_POOL_NAME,
            fingerprint=STUFF_POOL_FINGERPRINT,
            gomaestroorg_api_key=gomaestroorg_api_key,
        )
